-- Created by miere, version 1.0
-- CS2D version: beta 0.1.2.4
--
-- Puts top players in serverinfo.txt



-- do not parse junk data
rank_skipjunk = true

-- max players to write in MOTD.
-- 0.006 ms execution time on >80 players
rank_maxplayers = 3

-- string to write for each player
-- these words will be replaced at runtime:
--   #id#         - position in leaderboard 
--   #name#       - last used player name
--   #usgn#       - USGN id
--   #score#      - score (frags + reward for special actions like bomb defusing)
--   #frags#      - kills
--   #deaths#     - deaths
--   #kdratio#    - kills per death (#frags/#deaths, see rankformat_kdratio)
--   #sdratio#    - score per death (#score/#deaths, see rankformat_sdratio)
--   #time#       - time on server (see rankformat_time)
--
-- use "\n" to make a new line
-- this variable can be a table, a function or a string
--   function must return string
--   function can be inside table values
-- table & function example
--[[
	rank_pattern = {
		[1] = "the blaargh is #name#\n",           -- string for first player in leaderboard
		[2] = function()                           -- function for second player
			local words = {"such #frags# frags", "many #deaths# deaths",
				"very #kdratio# k/d", "wow #id# pos",}
			local ret = "#name# "
			for i = 1, #words-1 do
				local rnd = math.random(1, #words-1)
				print(rnd .. " " .. #words-1)
				ret = ret .. table.remove(words, rnd) .. " "
			end
			return ret .. "\n"
		end,  
		
        ["others"] = "crazy #name# at #id# pos\n", -- for others that don't have their strings
	}
--]]
rank_pattern = "#id#) #name#: #score# score, #frags# frags, #deaths# deaths (#sdratio# S/D)\n"

-- sort function:
--   you may replace condition with:
--     score, frags, deaths, usgn, kdratio, sdratio, time
--     (description above)
--     1) you should write your own func if you want compare names
--     2) id not available
--   also:
--     replace ">" with "<" if you want lower values first
--     replace "<" with ">" if you want higher values first
--   (or write your own func)
rank_sortfunction = 
	function(a1, a2)
		if (a1.frags+a1.score > a2.frags+a2.score) then
			return true
		end
	end

-- kdratio is score/deaths
-- format: http://www.cplusplus.com/reference/cstdio/printf/
rankformat_kdratio = 
	function(kdratio)
		return string.format("%.2f", kdratio)
	end

-- sdratio score/deaths
rankformat_sdratio =
	function(sdratio)
		return string.format("%.2f", sdratio)
	end

-- s is seconds
rankformat_time =
	function(s)
		return string.format("%.2dh %.2dm %.2ds", s/3600, s/60%60, s%60)
	end


addhook("mapchange", "onmapchange_updatemotd")
playerstats = {}

function int(str)
	local num = 0
	local len = #str
	for i = 1, len do
		num = num + string.byte(str, i) * 256^(len-i)
	end
	
	return num
end

-- userstats file format 
-- http://unrealsoftware.de/forum_posts.php?post=338621#post338627
function parsestats()
	local file = assert(io.open("sys/stats/userstats.dat", "rb"))
	file:seek("set", 17) -- skip header
	
	local content = file:read("*all")
	file:close()
	
	local current = 1
	local offset = 1
	
	while true do
		local invalid = false
		
		-- magic section 
		
		local pos = content:find("\r\n", offset)
		if pos == nil then break end
		
		playerstats[current] = {}
		local p = playerstats[current]
		
		p.name   = content:sub(offset, pos-1)
		
		offset = pos + 2
		
		p.usgn    = int(string.reverse(content:sub(offset, offset+3)))
		p.score   = int(string.reverse(content:sub(offset+4, offset+7)))
		p.frags   = int(string.reverse(content:sub(offset+8, offset+11)))
		p.deaths  = int(string.reverse(content:sub(offset+12, offset+15)))
		p.time    = int(string.reverse(content:sub(offset+16, offset+19))) -- SECONDS!
		p.kdratio = p.frags/p.deaths
		if p.deaths == 0 then p.kdratio = p.frags + 0.0 end
		p.sdratio = p.score/p.deaths
		if p.deaths == 0 then p.sdratio = p.score + 0.0 end
		
		offset = offset + 20
		
		-- magic section end
		
		--[[
		print(string.format("%s - usgn: %s, score: %s, frags: %s, deaths: %s, time: %s\n", 
			p.name, p.usgn, p.score, p.frags, p.deaths, p.time))
		--]]
	
		if rank_skipjunk then
			if (p.usgn   >= 1000000000 or
					p.score  >= 1000000000 or    
					p.frags  >= 1000000000 or
					p.deaths >= 1000000000 or
					p.time   == 4294967295 or
					p.time   == 0) then
				invalid = true
			end
		end
		
		if (invalid == true) then
			playerstats[current] = nil
		else
			current = current + 1
		end
	end
	
	return playerstats
end

function sortstats()
	if #playerstats == 0 then return end
	
	table.sort(playerstats, rank_sortfunction)
end

function formatstats(pattern, p)
	local data = {
		["#id#"]      = p.id,
		["#name#"]    = p.name,
		["#usgn#"]    = p.usgn,
		["#score#"]   = p.score,
		["#frags#"]   = p.frags,
		["#deaths#"]  = p.deaths,
		["#kdratio#"] = rankformat_kdratio(p.kdratio),
		["#sdratio#"] = rankformat_sdratio(p.sdratio),
		["#time#"]    = rankformat_time(p.time),
	}
	
	local usepattern
	if (type(rank_pattern) == "table") then
		if (pattern[p.id] ~= nil) then
			usepattern = pattern[p.id]
		else 
			usepattern = pattern["others"]
		end
	else
		usepattern = pattern
	end
	
	if (type(usepattern) == "function") then
		usepattern = usepattern() -- LUA STRONG AS POLAND
	end
	
	local ret = usepattern
	
	-- for each word #something#
	for w in string.gmatch(usepattern, "#%a+#") do
		if (data[w] ~= nil) then
			ret = string.gsub(ret, w, data[w])
		end
	end
	
	return ret
end

function updatemotd()
	local t = assert(io.open("sys/serverinfo.template.txt", "r"))
	local template = t:read("*all")
	t:close()
	
	local c = ""
	
	if (rank_maxplayers > #playerstats) then
		rank_maxplayers = #playerstats
	end
		
	for i = 1, rank_maxplayers do
		local p = playerstats[i]
		
		-- required info to format stats
		p.id = i;
		
		c = c .. formatstats(rank_pattern, p)
	end
	
	local motdcontent = template:gsub("{RANKMOTD}", c)
	
	local motd = assert(io.open("sys/serverinfo.txt", "w"))
	motd:write(motdcontent)
	motd:close()
end

function onmapchange_updatemotd(newmap)
	parsestats()
	sortstats()
	updatemotd()
end